/*
     Copyright (C) 2006  Georges Da Costa

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along
 with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>

#include <sys/soundcard.h>

#include <sys/sysinfo.h>
#include <time.h>

#include <sys/time.h>
#include  <stdio.h>
struct timeval  tv;
struct timezone  tz;

#include "toolbox.h"

//int total_ram = 0;
int total_battery = 0;

inline int max(int a, int b) { if(a>b) return a; return b;}
inline int min(int a, int b) { if(a>b) return b; return a;}

int cpufreq=0;
int get_cpufreq() {
  return cpufreq;
}
char *cpufreq_file=NULL;
void init_cpufreq(char *txt) {
  if(cpufreq_file != NULL) free(cpufreq_file);

  cpufreq_file=malloc(50+strlen(txt));
  sprintf(cpufreq_file,"/sys/devices/system/cpu/%s/cpufreq/scaling_governor", txt);
}
void eval_cpufreq() {
  FILE* temp_file;
  char buffer[6];
  temp_file = fopen(cpufreq_file,"r");
  if(temp_file == NULL) {
    cpufreq=-1;
    return;
  }
  fread(buffer, 5, 1, temp_file);
  buffer[5]=0;
  if(!strcmp(buffer,"power"))
    cpufreq=0;
  else if(!strcmp(buffer,"perfo"))
    cpufreq=2;
  else
    cpufreq=1;
  fclose(temp_file);
}
void set_cpufreq(int level) {
  FILE* temp_file;

  temp_file=fopen(cpufreq_file,"w");
  if(temp_file == NULL)
    return;

  cpufreq=level;
  if(level == 0)
    fwrite("powersave", 9, 1, temp_file);
  else if(level == 2) 
    fwrite("performance", 11, 1, temp_file);
  else
    fwrite("ondemand", 8, 1, temp_file);
  fclose(temp_file);
}

int status_temperature=0;
int get_temperature() {
  return status_temperature;
}
char *temperature_file=NULL;
void init_temperature(char *txt) {
  if(temperature_file != NULL) free(temperature_file);

  temperature_file=malloc(37+strlen(txt));
  sprintf(temperature_file,"/proc/acpi/thermal_zone/%s/temperature", txt);
}
void eval_temperature() {
  FILE* temp_file;
  char buffer[3];
  temp_file = fopen(temperature_file,"r");
  if(temp_file==NULL) {
    status_temperature = -1;
    return;
  }
  fseek(temp_file, 25, SEEK_CUR);
  fread(buffer,2,1,temp_file);
  fclose(temp_file);
  buffer[2]='\0';
  status_temperature = atoi(buffer);
}

void skip_line(FILE *f, int n) {
  unsigned int r = 0;
  int i;
  for(i=0;i<n;r=0,i++)
    while(r != 10)
      r=fgetc(f);
}

int status_load=0;
int sum_load=1;
int idle_load=1;
int get_load() {
  return status_load;
}
void eval_load() {
  FILE* temp_file;
  int a,b,c,d,e,f,g;

  temp_file=fopen("/proc/stat","r");
  if(temp_file == NULL) {
    status_load=0;
    return;
  }
  fscanf(temp_file,"cpu  %d %d %d %d %d %d %d\n",&a,&b,&c,&d,&e,&f,&g);
  fclose(temp_file);

  a = a+b+c+d+e+f+g;
  b = max(a-sum_load,1);
  g = d-idle_load;
  sum_load = a;
  idle_load = d;
  status_load = (100*(b-g))/b;
}

int status_vol = 0;
int status_pcm = 0;
int get_vol() {
  return status_vol;
}
int get_pcm() {
  return status_pcm;
}
void eval_sound() {
  int mixer_fd;
  int vol=0;
  mixer_fd = open("/dev/mixer",O_RDWR);
  status_pcm = status_vol = 0;
  if(mixer_fd == -1) {
    return;
  }

  if (ioctl(mixer_fd, SOUND_MIXER_READ_PCM, &vol) != -1)
    vol = vol / 257;
  status_pcm = vol;

  if (ioctl(mixer_fd, SOUND_MIXER_READ_VOLUME, &vol) != -1)
    vol = vol / 257;

  status_vol = vol;
  close(mixer_fd);
}
void set_sound(int decalage, int channel, int *status) {
  int mixer_fd;
  int volume;
  mixer_fd = open("/dev/mixer",O_RDWR);
  if(mixer_fd == -1) return;

  *status = max(0,min(decalage, 255));
  volume = (*status << 8) | *status;
  ioctl(mixer_fd, channel, &volume);
  close(mixer_fd);

}
void set_vol(int decalage) {
  set_sound(decalage, SOUND_MIXER_WRITE_VOLUME, &status_vol);
}
void set_pcm(int decalage) {
  set_sound(decalage, SOUND_MIXER_WRITE_PCM, &status_pcm);
}

struct tm *get_time() { 
  time_t now;
  now = time(NULL);
  return localtime(&now);
}

unsigned long int status_download=0;
unsigned long int status_upload=0;
double time_last_net;

int nb_interfaces=0;
int *download_interfaces=NULL;
int *upload_interfaces=NULL;
void eval_net() {
  FILE* temp_file;


  double time_current;
  double delta;
  gettimeofday(& tv , & tz );
  time_current = ((double)(tv.tv_sec%100))+ ((double)tv.tv_usec)/1000000;
  delta = time_current-time_last_net;
  if(delta < 0) delta+=100;
  time_last_net=time_current;


  status_download = 0;
  status_upload = 0;
  temp_file=fopen("/proc/net/dev","r");
  if(temp_file != NULL) {
    int pos=0;
    skip_line(temp_file,2);

    while(!feof(temp_file)) {
      unsigned long int up;
      unsigned long int dl;
      int nb = fscanf(temp_file, " %*[^:]:%lu %*d %*d %*d %*d %*d %*d %*d %lu " ,&dl, &up);
      if(nb!=2)
	break;
      pos++;
      if(pos > nb_interfaces) {
	nb_interfaces++;
	download_interfaces=realloc(download_interfaces,
				    nb_interfaces*sizeof(int));
	download_interfaces[nb_interfaces-1]=0;
	upload_interfaces=realloc(upload_interfaces,
				  nb_interfaces*sizeof(int));
	upload_interfaces[nb_interfaces-1]=0;
      }
      status_download += (dl-download_interfaces[pos-1])/delta;
      download_interfaces[pos-1] = dl;
      status_upload += (up-upload_interfaces[pos-1])/delta;
      upload_interfaces[pos-1] = up;
      
    }
    fclose(temp_file);
  }
}

unsigned long int get_upload() {
  return status_upload;
}
unsigned long int get_download() {
  return status_download;
}

int status_ac=0;
int get_ac() {
  return status_ac;
}
char *ac_file=NULL;
void init_ac(char *txt) {
  if(ac_file != NULL) free(ac_file);

  ac_file=malloc(29+strlen(txt));
  sprintf(ac_file,"/proc/acpi/ac_adapter/%s/state", txt);
}
void eval_ac() {
  FILE* temp_file;
  unsigned int r;
  temp_file=fopen(ac_file,"r");
  if(temp_file == NULL) {
    status_ac = FALSE;
    return;
  }
  fseek(temp_file, 26, SEEK_CUR);
  r = fgetc(temp_file);
  fclose(temp_file);
  status_ac = (r==110);
}

int status_battery = 0;
int get_battery() {
  if(total_battery == 0) return 0;
  return status_battery;
}
int rate_battery = 0;
int get_rate_battery() {
  if(total_battery == 0) return 0;
  return rate_battery;
}
char *battery_info=NULL;
char *battery_state=NULL;
void init_battery(char *txt) {
  if(battery_info != NULL) free(battery_info);
  if(battery_state != NULL) free(battery_state);

  battery_info=malloc(25+strlen(txt));
  sprintf(battery_info,"/proc/acpi/battery/%s/info", txt);

  battery_state=malloc(26+strlen(txt));
  sprintf(battery_state,"/proc/acpi/battery/%s/state", txt);
}
void eval_battery() {
  FILE* temp_file; 
  char buffer[8];
  
  if(total_battery == 0) {
    temp_file = fopen(battery_info,"r");
    if(temp_file != NULL) {
      fseek(temp_file, 29+25, SEEK_CUR);
      fread(buffer,7,1,temp_file);
      buffer[7]='\0';
      fclose(temp_file);
      total_battery =atoi(buffer);
      if(total_battery == 0) return;
    } else return;
  }

  temp_file=fopen(battery_state,"r");
  if(temp_file == NULL) {
    total_battery =0;
    return;
  }

  skip_line(temp_file,3);

  fscanf(temp_file, "present rate: %d mW\n", &rate_battery);
  rate_battery = (100*rate_battery)/total_battery;

  fscanf(temp_file, "remaining capacity: %d mWh\n", &status_battery);

  fclose(temp_file);
  status_battery = (100*status_battery)/total_battery;
}

/*
int status_ram=0;
int get_ram() {
  return status_ram;
}
void eval_ram() {
  struct sysinfo info;
  sysinfo(&info);
  status_ram = (100*(total_ram-(info.freeram/1024)))/total_ram;
}
*/

int maj_time=0;
void maj_captors(void *data) {
DEBUGIN("Maj captors");
  maj_time++;
  /* Tout ce qui se fait toutes les secondes : */
  eval_load();
  eval_sound();
  eval_ac();
  //  eval_ram();
  eval_cpufreq();

  /* Tout ce qui se fait une seconde sur deux : */
  if(maj_time%2==1) {
    eval_net();
  }

  /* Tout ce qui se fait une seconde sur quatre */
  if(maj_time%4 == 3) {
    eval_temperature();
  }

  /* Tout ce qui se fait une seconde sur dix */
  if(maj_time%10 == 3) {
    eval_battery();
  }

  /* Tout ce qui se fait une seconde sur 30 */
  /*
  if(maj_time%30 == 1) {
    eval_date();
  }
  */
DEBUGOUT("Maj captors");
}

void init_captor() {
  //  struct sysinfo info;

  /* Init de la mmoire */
  //  sysinfo(&info);
  //  total_ram = (info.totalram/1024);

  /* Init de la batterie */
  eval_battery();

  /* Init du download/upload */
  eval_net();
}

